"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.TextRenderer = void 0;
const variable_1 = require("petals-stem/dist/src/block/input/variable");
const list_1 = require("petals-stem/dist/src/block/input/list");
const input_1 = require("petals-stem/dist/src/block/input");
const block_1 = require("petals-stem/dist/src/block");
const renderer_1 = require("./renderer");
const chalk_1 = __importDefault(require("chalk"));
const broadcast_1 = require("petals-stem/dist/src/block/input/broadcast");
const color_1 = require("petals-stem/dist/src/block/input/color");
const manager_1 = require("../strings/manager");
const variable_2 = require("petals-stem/dist/src/block/field/variable");
const list_2 = require("petals-stem/dist/src/block/field/list");
const broadcast_2 = require("petals-stem/dist/src/block/field/broadcast");
const string_1 = require("petals-stem/src/block/input/string");
class TextRenderer extends renderer_1.Renderer {
    constructor(language = "English", colorLevel = 3) {
        super();
        this.stringsManager = new manager_1.StringsManager(language);
        this.chalk = new chalk_1.default.Instance({ level: colorLevel });
    }
    renderStack(b, dark) {
        let str = "";
        while (b) {
            if (b.getOpcode() === "__phantom__") {
                b = b.getNext();
                continue;
            }
            ;
            str += this.renderBlock(b, !dark, false) + "\n";
            b = b.getNext();
        }
        return str;
    }
    renderBlock(b, dark, includeParns = true) {
        if (b.getOpcode() === "argument_reporter_string_number" || b.getOpcode() === "argument_reporter_boolean") {
            return includeParns ? chalk_1.default.bgHex("#FF6680").bold.white(`( ${b.getField("VALUE").getValue()} )`) : chalk_1.default.bgHex("#FF6680").bold.white(` ${b.getField("VALUE").getValue()} `);
        }
        const args = this.stringsManager.getArgs(b);
        let m = {};
        for (const field of args.fields) {
            const contents = b.getField(field.substring(1));
            if (!contents) {
                m[field] = "Unset field (" + field + ")";
                continue;
            }
            m[field] = `${this.renderInput(contents, !dark)}`;
        }
        for (const field of args.inputs) {
            const contents = b.getInput(field.substring(1));
            if (!contents) {
                m[field] = "Unset input (" + field + ")";
                continue;
            }
            m[field] = `${this.renderInput(contents, !dark)}`;
        }
        for (const stack of args.stacks) {
            const headInput = b.getInput(stack.substring(1));
            if (!headInput) {
                m[stack] = "Unset stack (" + stack + ")";
                continue;
            }
            const rendered = this.renderStack(headInput.getTopLayer(), dark);
            m[stack] = " \n" + rendered.split("\n").slice(0, -1).map(line => this.wrapColorForCategory(b.getOpcode().split("_")[0], b.getOpcode(), "", !dark, false) + line).join("\n") + "\n" + this.wrapColorForCategory(b.getOpcode().split("_")[0], b.getOpcode(), " ", !dark, false);
        }
        return this.wrapColorForCategory(b.getOpcode().split("_")[0], b.getOpcode(), this.stringsManager.getForFormatted(b, m), !dark, includeParns);
    }
    renderInput(i, dark, includeParns = true) {
        const top = i instanceof input_1.Input ? i.getTopLayer() : i;
        if (top instanceof block_1.Block)
            return this.renderBlock(top, dark, includeParns);
        if (top instanceof variable_1.VariableInput || top instanceof variable_2.VariableField)
            return this.chalk.bgHex("#FF8C1A").white(includeParns ? `( ${top.getValue().getName()} )` : ` ${top.getValue().getName()} `);
        if (top instanceof list_1.ListInput || top instanceof list_2.ListField)
            return this.chalk.bgHex("#FF661A").white(includeParns ? `( ${top.getValue().getName()} )` : ` ${top.getValue().getName()} `);
        if (top instanceof broadcast_1.BroadcastInput || top instanceof broadcast_2.BroadcastField)
            throw new Error("TODO");
        if (top instanceof color_1.ColorInput)
            return this.chalk.bgWhite.black(` #${top.getValue()[0].toString(16).padStart(2, "0")}${top.getValue()[1].toString(16).padStart(2, "0")}${top.getValue()[2].toString(16).padStart(2, "0")} `);
        if (top instanceof string_1.StringInput)
            return this.chalk.bgWhite.black(includeParns ? `( "${top.getValue()}" )` : ` "${top.getValue()}" `);
        if (top)
            return this.chalk.bgWhite.black(includeParns ? `( ${top.getValue()} )` : ` ${top.getValue()} `);
        return this.chalk.bgWhite.black(includeParns ? `( ${this.chalk.grey("NULL")} )` : ` ${this.chalk.grey("NULL")} `);
    }
    wrapColorForCategory(category, opcode, str, dark, includeParns = true) {
        str = includeParns ? `( ${str} )` : ` ${str} `;
        switch (category) {
            case "argument":
            case "procedures": return chalk_1.default.bgHex(dark ? "#FA002A" : "#FF6680").bold.white(str);
            case "operator": return chalk_1.default.bgHex(dark ? "#368F36" : "#59C059").bold.white(str);
            case "control": return chalk_1.default.bgHex(dark ? "#C47C00" : "#FFAB19").bold.white(str);
            case "motion": return chalk_1.default.bgHex(dark ? "#00429E" : "#4c97FF").bold.white(str);
            case "event": return chalk_1.default.bgHex(dark ? "#B38600" : "#FFBF00").bold.white(str);
            case "looks": return chalk_1.default.bgHex(dark ? "#30008F" : "#9966FF").bold.white(str);
            case "data":
                if ((["data_setvariableto", "data_changevariableby", "data_showvariable", "data_hidevariable"]).includes(opcode)) {
                    return chalk_1.default.bgHex(dark ? "#C56200" : "#FF8C1A").bold.white(str);
                }
                else {
                    return chalk_1.default.bgHex(dark ? "#C54100" : "#FF661A").bold.white(str);
                }
            default: throw new Error("Unknown category " + category);
        }
    }
}
exports.TextRenderer = TextRenderer;
