"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.StringsManager = void 0;
const english_1 = require("./english");
const langGroup = {
    English: english_1.English,
};
class StringsManager {
    constructor(lang) {
        this.lang = lang;
    }
    getForRaw(block) {
        return langGroup[this.lang][block.getOpcode().toUpperCase()];
    }
    getArgs(block) {
        if (block.getOpcode() === "procedures_call" || block.getOpcode() === "procedures_prototype") {
            let p;
            if (block.getOpcode() === "procedures_call") {
                p = block.getPrototype();
            }
            else {
                p = block;
            }
            return {
                inputs: p.getArgumentIds().map(e => `i${e}`),
                fields: [],
                stacks: [],
            };
        }
        const res = langGroup[this.lang][block.getOpcode().toUpperCase()];
        const results = { inputs: [], fields: [], stacks: [] };
        for (let i = 0; i < res.length; i++) {
            if (res[i] === "%") {
                i++;
                const selector = res[i];
                let text = "";
                while (res[i] !== ";") {
                    if (i > res.length)
                        throw new Error("Invalid string format for opcode: " + block.getOpcode() + ", language: " + this.lang);
                    text += res[i];
                    i++;
                }
                results[selector === "i" ? "inputs" : selector === "s" ? "stacks" : "fields"].push(text);
            }
        }
        return results;
    }
    getForFormatted(block, args) {
        if (block.getOpcode() === "procedures_call" || block.getOpcode() === "procedures_prototype") {
            let p;
            if (block.getOpcode() === "procedures_call") {
                p = block.getPrototype();
            }
            else {
                p = block;
            }
            const argIds = p.getArgumentIds();
            const name = p.getProcCode();
            let result = "";
            for (let i = 0; i < name.length; i++) {
                if (name[i] === "%") {
                    i++;
                    console.log(args, argIds, "i" + argIds[0]);
                    result += args["i" + argIds.shift()];
                }
                else {
                    result += name[i];
                }
            }
            return result;
        }
        const res = langGroup[this.lang][block.getOpcode().toUpperCase()];
        let formatted = "";
        for (let i = 0; i < res.length; i++) {
            if (res[i] === "%") {
                let kind = "";
                i++;
                while (res[i] !== ";") {
                    if (i > res.length)
                        throw new Error("Invalid string format for opcode: " + block.getOpcode() + ", language: " + this.lang);
                    kind += res[i++];
                }
                formatted += args[kind];
                continue;
            }
            formatted += res[i];
        }
        return formatted;
    }
}
exports.StringsManager = StringsManager;
